;**************************************************************************
;*
;* FILE  ld2_dtv.asm
;* Copyright (c) 2008, 2009 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: ld2_dtv.asm,v 1.97 2010-04-02 16:54:27 tlr Exp $
;*
;* DESCRIPTION
;*   Loader II DTV side main code
;*
;*   frequencies: 985250 Hz (PAL), 1022730 (NTSC) and 1000000 MHz (1541)
;*
;*   PAL: 
;*     (985250/1000000)*68 = 66.99700
;*     i.e 67 C64/DTV cycles per 68 drive cycles
;*
;*   NTSC:
;*     (1022730/1000000)*44 = 45.00012 
;*     i.e 45 C64/DTV cycles per 44 drive cycles
;*
;******
	include	"macros.i"
	INFO	"--- start of file ld2_dtv.asm ---"		

	include	"ld2_zp_bss.asm"

; configuration options
MAX_TRACK	equ	40
ALLOW_IRQ	equ	1

; Use timer B of CIA 1 for pacing
PACE_CIA	equ	$dc00
;PACE_CIA	equ	$dd00

; Debugging options
COLOR_DEBUG	equ	0


; configuration macros
	mac	SEI_IF_ALLOW_IRQ
	if	ALLOW_IRQ
	sei
	endif
	endm

	mac	CLI_IF_ALLOW_IRQ
	if	ALLOW_IRQ
	cli
	endif
	endm
	
; debug macros
	mac	CDEBUG
	if	COLOR_DEBUG
	pha
	lda	#{1}
	sta	$d020
	pla
	endif
	endm
	mac	INC_D020
	if	COLOR_DEBUG
	inc	$d020
	endif
	endm
	mac	DEC_D020
	if	COLOR_DEBUG
	dec	$d020
	endif
	endm


	seg	code
;**************************************************************************
;*
;* NAME  prepare_dma
;*
;* DESCRIPTION
;*   Prepare loader for DMA
;*
;******
prepare_dma:
	SEI_IF_ALLOW_IRQ
	sac	#$99
	lda	#%00000011
	sac	#$00
	CLI_IF_ALLOW_IRQ
	jsr	wait_dma_blit	

; pre-clear DMA + blitter
	ldx	#$40
	lda	#0
pd_lp1:
	sta	$d300,x
	dex
	bpl	pd_lp1

; Setup DMA
	lda	#$80
	sta	$d302		; Src MSB
	if	[dma_dd00_buffer_real>>16]
	lda	#[dma_dd00_buffer_real>>16] | $40
	else
	lsr
	endif
	sta	$d305		; Dest MSB
	lda	#$dd
	sta	$d301		; Src MidSB
	lda	#[dma_dd00_buffer_real>>8] & $ff
	sta	$d304		; Dest MidSB
	lda	#1
	sta	$d308		; Dest step LSB
	lda	#$1a
	sta	$d30b		; DMA length MSB

; Setup Blitter
	ifconst	HAVE_SETUP_BLITTER_HOOK
	SETUP_BLITTER_HOOK
	endif

; Setup pacing timer
	lda	#%00000000	; stop timer B
	sta	PACE_CIA+$0f
	lda	#%00000010	; disable timer B interrupt
	sta	PACE_CIA+$0d
	lda	PACE_CIA+$0d	; Acknowledge
	lda	#$ff		; Timer B = $ffff
	sta	PACE_CIA+$06
	sta	PACE_CIA+$07

; Sample vic-bank
	lda	$dd00
	and	#%00000011
	sta	bank_zp

	asl
	asl
	eor	bank_zp
; Acc = [DEF_DD00 << 2] ^ [DEF_DD00 << 0] 
	sta	fixc1
	sta	fixc2
	sta	fixc3
	sta	fixc4
	sta	fixc5

	asl
	asl
	eor	bank_zp
; Acc = [DEF_DD00 << 4] ^ [DEF_DD00 << 2] ^ [DEF_DD00 << 0]
	sta	fixb1
	sta	fixb2
	sta	fixb3
	sta	fixb4
	sta	fixb5

	asl
	asl
	eor	bank_zp
; Acc = [DEF_DD00 << 6] ^ [DEF_DD00 << 4] ^ [DEF_DD00 << 2] ^ [DEF_DD00 << 0]
	sta	fixa1
	rts

;**************************************************************************
;*
;* NAME  read_track
;*
;* DESCRIPTION
;*   read one track to memory
;*
;******
read_track:
	lda	track_zp
	jsr	put_byte

	lda	track_zp
	jsr	calc_num_sectors
	sta	num_zp
	sta	left_zp
	tax
	lda	#0
	sta	err_zp
rt_lp1:
	sta	read_tab-1,x
	dex
	bne	rt_lp1

read_blocks:	
rt_lp2:
	jsr	get_block
	ifconst	HAVE_SECTOR_HOOK
	pha
	SECTOR_HOOK
	pla
	endif
	cmp	#$01		; $01  Everything OK
	bne	rt_fl1		; no, skip to error handler
; sector ok, mark as read
	ldx	sector_zp
	inc	read_tab,x

rt_skp1:
	dec	left_zp
	bne	rt_lp2

	lda	err_zp
	bne	rt_fl2
	clc
	rts

rt_fl1:	
; check for errors that do not abort transfer.
	cmp	#$05		; $05  Checksum error in data block
	bne	rt_fl2		; no, terminal error, skip out
; non-terminal error, flag error and continue
	inc	err_zp	
	jmp	rt_skp1

rt_fl2:
	if	0
; terminal error.
	sec
	rts
	endif

re_read_track:
	lda	#$83
	jsr	put_byte

	ldy	#0
	sty	err_zp
	sty	left_zp
rrt_lp1:
	clc
	lda	read_tab,y
	bne	rrt_skp1
	inc	left_zp
	sec
rrt_skp1:
	ror	tmp_zp
	iny
	tya
	and	#7
	bne	rrt_skp2
	lda	tmp_zp
	jsr	put_byte
rrt_skp2:	
	cpy	num_zp
	bne	rrt_lp1
rrt_lp2:
	lsr	tmp_zp
	iny
	cpy	#24
	bne	rrt_lp2
	lda	tmp_zp
	jsr	put_byte


	jmp	read_blocks
	
;**************************************************************************
;*
;* NAME  calc_num_sectors
;*
;* DESCRIPTION
;*   Acc=track
;*   returns number of sectors in Acc.
;*
;******
calc_num_sectors:
	ldx	#4
cns_lp1:
	dex
	cmp	cns_tracktab,x
	bcs	cns_lp1
	lda	cns_numtab,x
	rts
cns_tracktab:
	dc.b	MAX_TRACK+1, 30+1, 24+1, 17+1
cns_numtab:
	dc.b	17,   18,   19,   21

;**************************************************************************
;*
;* NAME  wait_dma_blit
;*
;* DESCRIPTION
;*   Wait until DMA and blitter has finished
;*
;******
wait_dma_blit:
wdb_lp1:
	lda	$d31f
	ora	$d33f
	lsr
	bcs	wdb_lp1
	rts

;**************************************************************************
;*
;* NAME  get_block
;*
;* DESCRIPTION
;*   Get one disk block using DMA and simultaneously decode it.
;*
;******
	if	1
TIME_CYCLE	equ	126
TIME_INITIAL	equ	TIME_CYCLE*[26+12]
ALIGN_SPACING	equ	5
ALIGN_TIME_UP	equ	1
	else
TIME_CYCLE	equ	131
TIME_INITIAL	equ	TIME_CYCLE*[26+12]
ALIGN_SPACING	equ	7
ALIGN_TIME_UP	equ	0
	endif		

PACE_MARGIN_1	equ	1	; 1*256 cycles
PACE_MARGIN_2	equ	2	; 2*256 cycles

	mac	SETTIME
_TIME	set	{1}
	endm
	mac	ADDTIME
_TIME	set	_TIME + {1}
	endm

	
get_block:
; wait until DMA and blitter are finished.
	jsr	wait_dma_blit

gbl_lp7:
	bit	$dd00		; DATA=0?
	bpl	gbl_lp7		; yes, drive busy
	bvs	gbl_skp10	; CLK=1, DATA=1, ready!
; CLK=0, DATA=1, fail error
	jmp	gbl_fl2

gbl_skp10:
; flag ready
	lda	bank_zp
	tax
	ora	#%00010000
	ldy	#8
	CDEBUG	1
	SEI_IF_ALLOW_IRQ
	nopalign 1
	sta	$dd00		; CLK=0
dly:
	nop
	dey
	bne	dly
	nop
	stx	$dd00		; CLK=1
	lda	#%00011001	; start timer B, one shot, force load
	ldy	#%00001101	; Source Dir=pos, Dest Dir=pos, Force Start=1
	nop
	sta	PACE_CIA+$0f
	nop
	sty	$d31f
	CLI_IF_ALLOW_IRQ
	CDEBUG	13

	lda	#0
	sta	ptr2_frac_zp
	lda	#<dma_dd00_buffer
	sta	ptr2_zp
	lda	#>dma_dd00_buffer
	sta	ptr2_zp+1

; 	CDEBUG	6
	lda	#$ff-PACE_MARGIN_1	; PACE_MARGIN_1*256 cycles head start
gbl_lp3:
	cmp	PACE_CIA+$07
	bcc	gbl_lp3
; 	CDEBUG	5

	nopalign 2
; find start
	ldy	#0
dcd_lp1:
	iny
	lda	(ptr2_zp),y
	and	#$c0
	cmp	#$c0
	bne	dcd_lp1
	sty	tmp_zp
	iny
	iny
	iny
	lda	(ptr2_zp),y
	and	#$c0
	cmp	#$c0
	beq	dcd_skpa
	dey
	dec	tmp_zp
	lda	ptr2_frac_zp
	sec
	sbc	#TIME_CYCLE
	sta	ptr2_frac_zp
dcd_skpa:
	iny
	iny
	iny
	iny
	lda	(ptr2_zp),y
	and	#$c0
	cmp	#$c0
	bne	dcd_skpb
	dec	tmp_zp
	lda	ptr2_frac_zp
	sec
	sbc	#TIME_CYCLE
	sta	ptr2_frac_zp
dcd_skpb:
	lda	tmp_zp
	sta	ptr2_zp

	lda	ptr2_frac_zp
	clc
	adc	#<TIME_INITIAL		
	sta	ptr2_frac_zp
	lda	ptr2_zp
	adc	#>TIME_INITIAL
	sta	ptr2_zp

	SETTIME	$80		; fake rounding
;--- GCR0
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 76------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	asl			; 6-------,7
	rol			; -------7,6
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54-----7,6
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 4-----76,5
	rol			; -----765,4
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 32---765,4
				; --xyxyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 2---7654,3
	rol			; ---76543,2
; acc= GCR0 hi
fixa1	equ	.+1
	eor	#$00		; xyxyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10-76543,2
				; --------,-
	ADDTIME	TIME_CYCLE*12
	rol			; 0-765432,1
	rol			; -7654321,0
	rol			; 76543210,-
; 	ldy	#_TIME/256
; 	eor	(ptr2_zp),y	; skip the next 2 bits as this isn't GCR.
	ADDTIME	TIME_CYCLE*8	; i.e bits 76 of the next byte
; acc=GCR0 lo
	sta	sector_zp
	clc
	adc	#>track_buffer
	sta	dcd_sm1+2

	ldx	#0
dcd_lp2:

;--- GCR1
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 98------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	asl			; 8-------,9
	rol			; -------9,8
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 76-----9,8
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 6-----98,7
	rol			; -----987,6
fixb1	equ	.+1
	eor	#$00		; --xyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54---987,6
				; --------,-
	ADDTIME	TIME_CYCLE*12
	rol			; 4---9876,5
	rol			; ---98765,4
; acc= GCR1 hi
	nopalign 1
	sta	htab,x

	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 32------,4
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 2------4,3
	rol			; ------43,2
fixc1	equ	.+1
	eor	#$00		; ----xyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10----43,2
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 0----432,1
	rol			; ----4321,0
	rol			; ---43210,-
; acc=GCR1 lo
	nopalign 0
	inx
	sta	ltab-1,x

;--- GCR2
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 98------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	asl			; 8-------,9
	rol			; -------9,8
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 76-----9,8
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*12
	rol			; 6-----98,7
	rol			; -----987,6
fixb2	equ	.+1
	eor	#$00		; --xyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54---987,6
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 4---9876,5
	rol			; ---98765,4
; acc= GCR2 hi
	nopalign 1
	sta	htab,x

	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 32------,4
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 2------4,3
	rol			; ------43,2
fixc2	equ	.+1
	eor	#$00		; ----xyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10----43,2
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 0----432,1
	rol			; ----4321,0
	rol			; ---43210,-
; acc=GCR2 lo
	nopalign 0
	inx
	sta	ltab-1,x

;--- GCR3
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 98------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*12
	asl			; 8-------,9
	rol			; -------9,8
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 76-----9,8
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 6-----98,7
	rol			; -----987,6
fixb3	equ	.+1
	eor	#$00		; --xyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54---987,6
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 4---9876,5
	rol			; ---98765,4
; acc= GCR3 hi
	nopalign 1
	sta	htab,x

	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 32------,4
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 2------4,3
	rol			; ------43,2
fixc3	equ	.+1
	eor	#$00		; ----xyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10----43,2
				; --------,-
;	ADDTIME	TIME_CYCLE*12	
	rol			; 0----432,1
	rol			; ----4321,0
	rol			; ---43210,-
; acc=GCR3 lo
	nopalign 0
	inx
	sta	ltab-1,x

	nopalign 2
	lda	(ptr2_zp),y
	ldy	#_TIME/256+ALIGN_SPACING
	cmp	(ptr2_zp),y	
	if	ALIGN_TIME_UP
	bne	dcd_skp1
	lda	ptr2_frac_zp
	clc
	adc	#TIME_CYCLE
	nopalign 2
	sta	ptr2_frac_zp
	bcc	dcd_skp1
	inc	ptr2_zp
	bne	dcd_skp1
	inc	ptr2_zp+1
	else
	beq	dcd_skp1
	lda	ptr2_frac_zp
	sec
	sbc	#TIME_CYCLE
	nopalign 2
	sta	ptr2_frac_zp
	nopalign 2
	lda	ptr2_zp
	sbc	#0
	sta	ptr2_zp
	bcc	dcd_skp1
	dec	ptr2_zp+1
	endif
dcd_skp1:	

 	ADDTIME	TIME_CYCLE*14
	ADDTIME	TIME_CYCLE*12

	nopalign 1
	clc
	lda	ptr2_frac_zp
	adc	#<[_TIME-$80]
	sta	ptr2_frac_zp
	nopalign 2
	lda	ptr2_zp
	adc	#>[_TIME-$80]
	sta	ptr2_zp
	bcc	dcd_skp2
	inc	ptr2_zp+1
dcd_skp2:

	SETTIME	$80		; fake rounding
;--- GCR0
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 98------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	asl			; 8-------,9
	rol			; -------9,8
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 76-----9,8
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 6-----98,7
	rol			; -----987,6
fixb4	equ	.+1
	eor	#$00		; --xyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54---987,6
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 4---9876,5
	rol			; ---98765,4
; acc= GCR0 hi
	nopalign 1
	sta	htab,x

	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 32------,4
				; ------xy,-
	ADDTIME	TIME_CYCLE*12
	rol			; 2------4,3
	rol			; ------43,2
fixc4	equ	.+1
	eor	#$00		; ----xyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10----43,2
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 0----432,1
	rol			; ----4321,0
	rol			; ---43210,-
; acc=GCR0 lo
	nopalign 1
	sta	ltab,x
	inx
	beq	dcd_ex1

; stall processing until [[$ff-PACE_MARGIN_2]-[>ptr2_zp->dma_dd00_buffer]]*2
; this works because ptr2_zp actually counts cycles*2.
; 	CDEBUG	6		
	nopalign 3
	sec
	lda	#[[$ff-PACE_MARGIN_2]+[>dma_dd00_buffer]]&$ff
	sbc	ptr2_zp+1
	asl
	nopalign 1
gbl_lp4:
	cmp	PACE_CIA+$07
	bcc	gbl_lp4
; 	CDEBUG	5
	
	nopalign 1
	jmp	dcd_lp2
dcd_ex1:


;--- GCR1
	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 98------,-
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	asl			; 8-------,9
	rol			; -------9,8
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 76-----9,8
				; ----xyxy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 6-----98,7
	rol			; -----987,6
fixb5	equ	.+1
	eor	#$00		; --xyxyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 54---987,6
				; --------,-
	ADDTIME	TIME_CYCLE*12
	rol			; 4---9876,5
	rol			; ---98765,4
; acc= GCR1 hi
	tax

	nopalign 0
	ldy	#_TIME/256
	lda	(ptr2_zp),y	; 32------,4
				; ------xy,-
	ADDTIME	TIME_CYCLE*8
	rol			; 2------4,3
	rol			; ------43,2
fixc5	equ	.+1
	eor	#$00		; ----xyxy ^
				; ------xy =
	nopalign 0
	ldy	#_TIME/256
	eor	(ptr2_zp),y	; 10----43,2
				; --------,-
	ADDTIME	TIME_CYCLE*8
	rol			; 0----432,1
	rol			; ----4321,0
	rol			; ---43210,-
; acc=GCR1 lo
	tay
	lda	gcr_ltab,y
	ora	gcr_htab,x
	sta	chksum_zp
	
	ldx	#0
dcd_lp3:
	nopalign 1
	ldy	ltab,x
	nopalign 1
	lda	gcr_ltab,y
	nopalign 1
	ldy	htab,x
	nopalign 1
	ora	gcr_htab,y
	nopalign 1
dcd_sm1:
	sta.wx	$0000,x
	nopalign 2
	eor	chksum_zp
	inx
	nopalign 2
	sta	chksum_zp
	bne	dcd_lp3

	ldx	chksum_zp
	bne	gbl_fl3
	lda	#$01		; $01  Everything OK
	clc
	rts

gbl_fl2:
	jsr	get_byte	; get error code from drive
	dc.b	$2c
gbl_fl3:
	lda	#$05		; $05  Checksum error in data block
	sec
	rts

;**************************************************************************
;*
;* NAME  gcr_ltab, gcr_htab
;*
;* DESCRIPTION
;*   gcr_[l|h]tab:
;*     0,   0,   0,   0,   0,   0,   0,   0 
;*     0,   $08, $00, $01, 0,   $0c, $04, $05
;*     0,   0,   $02, $03, 0,   $0f, $06, $07
;*     0,   $09, $0a, $0b, 0,   $0d, $0e, 0
;*
;******
gcr_ltab	equ	.-9
 	dc.b	     $08, $00, $01, 0,   $0c, $04, $05
	dc.b	0,   0,   $02, $03, 0,   $0f, $06, $07
	dc.b	0,   $09, $0a, $0b, 0,   $0d, $0e
gcr_htab	equ	.-9
	dc.b	     $80, $00, $10, 0,   $c0, $40, $50 
	dc.b	0,   0,   $20, $30, 0,   $f0, $60, $70
	dc.b	0,   $90, $a0, $b0, 0,   $d0, $e0

;**************************************************************************
;*
;* NAME  error_handler
;*
;* DESCRIPTION
;*   Handle error by just dumping the status to the screen
;*
;******
error_handler:
	sta	$0428
	jsr	get_byte
	sta	$0429
	lda	#$81		; motor off
	jsr	put_byte
	sei
	lda	#$02
	nopalign 0
eh_lp1
	sta	$d020
	eor	#8
	bra	eh_lp1
	
	subroutine	put_byte
;**************************************************************************
;*
;* NAME  put_byte
;*
;* DESCRIPTION
;*   Send byte to drive using slow 1-bit protocol
;*
;******
put_byte:
	DEC_D020
	sta	shift_zp

	SEI_IF_ALLOW_IRQ
	sac	#$99
	pha
	lda	#$00
	sac	#$00
	CLI_IF_ALLOW_IRQ

.pb_lp1:
	bit	$dd00		; DATA=0?
	bpl	.pb_lp1		; yes, drive busy
	bvc	.pb_fl1		; CLK=0, DATA=1, exit error
	lda	bank_zp
	sta	.pb_sm2+1
	ora	#%00010000
	sta	.pb_sm1+1

	ldx	#4
.pb_lp2:
.pb_sm1:
	lda	#$00
	ror	shift_zp
	bcc	.pb_skp1
	ora	#%00100000
.pb_skp1:
	sta	$dd00		; CLK=0
	jsr	.pb_twelve
.pb_sm2:
	lda	#$00
	ror	shift_zp
	bcc	.pb_skp2
	ora	#%00100000
.pb_skp2:
	sta	$dd00		; CLK=1
	jsr	.pb_twelve
	dex
	bne	.pb_lp2
	lda	bank_zp
	sta	$dd00
	SEI_IF_ALLOW_IRQ
	sac	#$99
	pla
	sac	#$00
	CLI_IF_ALLOW_IRQ
	INC_D020
.pb_twelve:
	rts
.pb_fl1:
	lda	#0
	jmp	error_handler

	subroutine	get_byte
;**************************************************************************
;*
;* NAME  get_byte
;*
;* DESCRIPTION
;*   get byte from drive using slow 1-bit protocol
;*
;******
get_byte:
	DEC_D020
	SEI_IF_ALLOW_IRQ
	sac	#$99
	pha
	lda	#$00
	sac	#$00
	CLI_IF_ALLOW_IRQ

.gb_lp1:
	bit	$dd00		; DATA=0?
	bpl	.gb_lp1		; yes, drive busy

	lda	bank_zp
	sta	.gb_sm2+1
	ora	#%00010000
	sta	.gb_sm1+1
	eor	#%00110000
	sta	$dd00		; DATA=0
	nop
	eor	#%00100000
	sta	$dd00		; DATA=1
	jsr	.gb_twelve
	ldx	#4
.gb_lp2:
	lda	$dd00
	asl
	ror	shift_zp
.gb_sm1:
	lda	#%00010011
	sta	$dd00		; CLK=0
	jsr	.gb_twelve
	jsr	.gb_twelve
	lda	$dd00
	asl
	ror	shift_zp
.gb_sm2:
	lda	#%00000011
	sta	$dd00		; CLK=1
	jsr	.gb_twelve
	jsr	.gb_twelve
	dex
	bne	.gb_lp2
	SEI_IF_ALLOW_IRQ
	sac	#$99
	pla
	sac	#$00
	CLI_IF_ALLOW_IRQ
	INC_D020
	lda	shift_zp
	eor	#$ff
.gb_twelve:
	rts

; eof
